/*
 * Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
 * Copyright (c) 2015, 2022, Loongson Technology. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

#include "precompiled.hpp"
#include "register_loongarch.hpp"

const int ConcreteRegisterImpl::max_gpr = RegisterImpl::number_of_registers * RegisterImpl::max_slots_per_register;
const int ConcreteRegisterImpl::max_fpr = ConcreteRegisterImpl::max_gpr +
                                                                  FloatRegisterImpl::number_of_registers * FloatRegisterImpl::max_slots_per_register;


const char* RegisterImpl::name() const {
  const char* names[number_of_registers] = {
    "zero", "ra", "tp", "sp", "a0/v0", "a1/v1", "a2", "a3",
    "a4", "a5", "a6", "a7", "t0", "t1", "t2", "t3",
    "t4", "t5", "t6", "t7", "t8", "x", "fp", "s0",
    "s1", "s2", "s3", "s4", "s5", "s6", "s7", "s8"
  };
  return is_valid() ? names[encoding()] : "noreg";
}

const char* FloatRegisterImpl::name() const {
  const char* names[number_of_registers] = {
    "f0",  "f1",   "f2",  "f3",   "f4",  "f5",   "f6",  "f7",
    "f8",  "f9",  "f10", "f11",  "f12", "f13",  "f14", "f15",
    "f16", "f17",  "f18", "f19", "f20", "f21",  "f22", "f23",
    "f24", "f25",  "f26", "f27",  "f28", "f29",  "f30", "f31",
  };
  return is_valid() ? names[encoding()] : "fnoreg";
}

const char* ConditionalFlagRegisterImpl::name() const {
  const char* names[number_of_registers] = {
    "fcc0",  "fcc1",   "fcc2",  "fcc3",   "fcc4",  "fcc5",   "fcc6",  "fcc7",
  };
  return is_valid() ? names[encoding()] : "fccnoreg";
}
